ChannelSelection

Type Alias ChannelSelection 

pub type ChannelSelection = ChannelSelectionImpl<MAX_CHANNELS, WORDS>;
Compact representation of an audio channel selection

Selected channel indices are encoded using a bit array, leading to a very small memory footprint. A channel selection is meant to be copied wherever it is needed. It includes methods for checking and modifying a channel selection status, as well as iterating over selected channel indices.

In the context of audio processing, a channel selection is synonymous with a “channel connection” between input and output buffers.

Aliased Type

pub struct ChannelSelection { /* private fields */ }