Processor

Trait Processor 

pub trait Processor<T: Sample> {
    // Required method
    fn process(&mut self, context: &mut ProcessingContext<'_, T>);
}
Trait for audio processing nodes

A processor implements the process method that operates directly on audio data provided through the ProcessingContext.

Required Methods

fn process(&mut self, context: &mut ProcessingContext<'_, T>)

Processes audio data from the input buffer to the output buffer of the provided context

Implementors